@@ -199,7 +199,7 @@ char * h_escape_string(const struct _h_connection * conn, const char * unsafe);
  * @param conn the connection to the database
  * @param unsafe the string to escape
  * @return a heap-allocated string
- * returned value must be h_h_free'd after use
+ * returned value must be h_free'd after use
  */
 char * h_escape_string_with_quotes(const struct _h_connection * conn, const char * unsafe);
 
@@ -316,8 +316,8 @@ int h_query_select_json(const struct _h_connection * conn, const char * query, j
  * A j_query has the following form
  * {
  *   "table": "table_name"             // String, mandatory, the table name where the query is executed
- *   "columns": ["col1", "col2"]       // Array of strings, available for h_select, optional. If not specified,will be used
- *   "order_by": "col_name [asc|desc]" // String, available for h_select, specify the order by clause, optional
+ *   "columns": ["col1", "col2"]       // Array of strings, available for h_select, optional. If not specified, * will be used, the columns values are not escaped by the library
+ *   "order_by": "col_name [asc|desc]" // String, available for h_select, specify the order by clause, optional, the value is not escaped by the library
  *   "limit": integer_value            // Integer, available for h_select, specify the limit value, optional
  *   "offset"                          // Integer, available for h_select, specify the limit value, optional but available only if limit is set
  *   "values": [{                      // json object or json array of json objects, available for h_insert, mandatory, specify the values to update
@@ -411,6 +411,25 @@ int h_update(const struct _h_connection * conn, const json_t * j_query, char **
  */
 int h_delete(const struct _h_connection * conn, const json_t * j_query, char ** generated_query);
 
+/**
+ * h_build_where_clause
+ * Generates a where clause based on the pattern and the values given
+ * @param conn the connection to the database
+ * @param pattern the pattern to build the where clause
+ * the pattern variables available are the following:
+ * - %s: a string value to escape with quotes
+ * - %S: a string value to escape without quotes
+ * - %c: a string value not to escape with quotes
+ * - %C: a string value not to escape without quotes
+ * - %d: an integer value in json_int_t format
+ * - %f: a double value
+ * - %j: a json_t value, the value must be of the type JSON_INTEGER, JSON_REAL or JSON_STRING, string values will be escaped with quotes
+ * - %%: the value '%'
+ * @return a heap-allocated string
+ * returned value must be h_free'd after use
+ */
+char * h_build_where_clause(const struct _h_connection * conn, const char * pattern, ...);
+
 /**
  * @}
  */
@@ -525,6 +544,16 @@ long long int h_last_insert_id_sqlite(const struct _h_connection * conn);
 
 /**
  * h_exec_query_sqlite
+ * legacy function name, didn't realize the typo until now...
+ * redirect to h_execute_query_sqlite
+ * @param conn the connection to the database
+ * @param query the SQL query to execute
+ * @return H_OK on success
+ */
+int h_exec_query_sqlite(const struct _h_connection * conn, const char * query);
+
+/**
+ * h_execute_query_sqlite
  * Execute a query on a sqlite connection
  * This is an internal function, you should use h_exec_query instead
  * Should not be executed by the user because all parameters are supposed to be correct
@@ -533,7 +562,7 @@ long long int h_last_insert_id_sqlite(const struct _h_connection * conn);
  * @param query the SQL query to execute
  * @return H_OK on success
  */
-int h_exec_query_sqlite(const struct _h_connection * conn, const char * query);
+int h_execute_query_sqlite(const struct _h_connection * conn, const char * query);
 
 /**
  * h_execute_query_json_sqlite
