@@ -19,21 +19,21 @@
 
 #define UNSAFE_STRING "un'safe' (\"string\")#!/$%*];"
 
-#define SELECT_DATA_1 "SELECT integer_col, string_col, date_col FROM test_table WHERE integer_col = 1"
-#define SELECT_DATA_2 "SELECT integer_col, string_col, date_col FROM test_table WHERE integer_col = 2"
-#define SELECT_DATA_ERROR "SELECT integer_col, string_col, date_col FROM test_table WHERE integer_col = 'error'"
+#define SELECT_DATA_1 "SELECT integer_col, double_col, string_col, date_col FROM test_table WHERE integer_col = 1"
+#define SELECT_DATA_2 "SELECT integer_col, double_col, string_col, date_col FROM test_table WHERE integer_col = 2"
+#define SELECT_DATA_ERROR "SELECT integer_col, double_col, string_col, date_col FROM test_table WHERE integer_col = 'error'"
 #define SELECT_DATA_ALL "SELECT * FROM test_table"
 
-#define INSERT_DATA_1 "INSERT INTO test_table (integer_col, string_col, date_col) VALUES (1, 'value1', date('now'))"
-#define INSERT_DATA_2 "INSERT INTO test_table (integer_col, string_col, date_col) VALUES (2, 'value2', strftime('%s','2016-06-22 00:52:56'))"
-#define INSERT_DATA_ERROR "INSERT INTO test_table (integer_col, string_col, date_col) VALUES ('error', 'value error', date('now'))"
+#define INSERT_DATA_1 "INSERT INTO test_table (integer_col, double_col, string_col, date_col) VALUES (1, 4.2, 'value1', date('now'))"
+#define INSERT_DATA_2 "INSERT INTO test_table (integer_col, double_col, string_col, date_col) VALUES (2, 5.4, 'value2', strftime('%s','2016-06-22 00:52:56'))"
+#define INSERT_DATA_ERROR "INSERT INTO test_table (integer_col, double_col, string_col, date_col) VALUES ('error', 'error_double', 'value error', date('now'))"
 
 #define DELETE_DATA_1 "DELETE FROM test_table WHERE integer_col = 1"
 #define DELETE_DATA_2 "DELETE FROM test_table WHERE integer_col = 2"
 #define DELETE_DATA_ERROR "DELETE FROM test_table WHERE wrong_table = 1"
 #define DELETE_DATA_ALL "DELETE FROM test_table"
 
-#define UPDATE_DATA_1 "UPDATE test_table SET string_col='new value1' WHERE integer_col = 1"
+#define UPDATE_DATA_1 "UPDATE test_table SET string_col='new value1', double_col=5.4 WHERE integer_col = 1"
 
 #define WHERE_CLAUSE_NO_FORMAT "And then, nothing happened"
 
@@ -164,15 +164,17 @@ START_TEST(test_hoel_insert)
   ck_assert_int_eq(h_query_insert(conn, NULL), H_ERROR_PARAMS);
   ck_assert_int_eq(h_query_select(conn, SELECT_DATA_1, &result), H_OK);
   ck_assert_int_eq(result.nb_rows, 1);
-  ck_assert_int_eq(result.nb_columns, 3);
+  ck_assert_int_eq(result.nb_columns, 4);
   ck_assert_int_eq(result.data[0][0].type, HOEL_COL_TYPE_INT);
   ck_assert_int_eq(((struct _h_type_int *)result.data[0][0].t_data)->value, 1);
-  ck_assert_int_eq(result.data[0][1].type, HOEL_COL_TYPE_TEXT);
-  ck_assert_str_eq(((struct _h_type_text *)result.data[0][1].t_data)->value, "value1");
+  ck_assert_int_eq(result.data[0][1].type, HOEL_COL_TYPE_DOUBLE);
+  ck_assert_double_eq(((struct _h_type_double *)result.data[0][1].t_data)->value, 4.2);
+  ck_assert_int_eq(result.data[0][2].type, HOEL_COL_TYPE_TEXT);
+  ck_assert_str_eq(((struct _h_type_text *)result.data[0][2].t_data)->value, "value1");
   ck_assert_int_eq(h_clean_result(&result), H_OK);
   ck_assert_int_eq(h_query_select(conn, SELECT_DATA_ERROR, &result), H_OK);
   ck_assert_int_eq(result.nb_rows, 0);
-  ck_assert_int_eq(result.nb_columns, 3);
+  ck_assert_int_eq(result.nb_columns, 4);
   ck_assert_int_eq(h_clean_result(&result), H_OK);
   ck_assert_int_eq(h_query_delete(conn, DELETE_DATA_1), H_OK);
   ck_assert_int_eq(h_close_db(conn), H_OK);
@@ -190,16 +192,18 @@ START_TEST(test_hoel_update)
   ck_assert_int_eq(h_query_insert(conn, NULL), H_ERROR_PARAMS);
   ck_assert_int_eq(h_query_select(conn, SELECT_DATA_1, &result), H_OK);
   ck_assert_int_eq(result.nb_rows, 1);
-  ck_assert_int_eq(result.nb_columns, 3);
+  ck_assert_int_eq(result.nb_columns, 4);
   ck_assert_int_eq(((struct _h_type_int *)result.data[0][0].t_data)->value, 1);
-  ck_assert_str_eq(((struct _h_type_text *)result.data[0][1].t_data)->value, "value1");
+  ck_assert_double_eq(((struct _h_type_double *)result.data[0][1].t_data)->value, 4.2);
+  ck_assert_str_eq(((struct _h_type_text *)result.data[0][2].t_data)->value, "value1");
   ck_assert_int_eq(h_clean_result(&result), H_OK);
   ck_assert_int_eq(h_query_update(conn, UPDATE_DATA_1), H_OK);
   ck_assert_int_eq(h_query_select(conn, SELECT_DATA_1, &result), H_OK);
   ck_assert_int_eq(result.nb_rows, 1);
-  ck_assert_int_eq(result.nb_columns, 3);
+  ck_assert_int_eq(result.nb_columns, 4);
   ck_assert_int_eq(((struct _h_type_int *)result.data[0][0].t_data)->value, 1);
-  ck_assert_str_eq(((struct _h_type_text *)result.data[0][1].t_data)->value, "new value1");
+  ck_assert_double_eq(((struct _h_type_double *)result.data[0][1].t_data)->value, 5.4);
+  ck_assert_str_eq(((struct _h_type_text *)result.data[0][2].t_data)->value, "new value1");
   ck_assert_int_eq(h_clean_result(&result), H_OK);
   ck_assert_int_eq(h_query_delete(conn, DELETE_DATA_1), H_OK);
   ck_assert_int_eq(h_close_db(conn), H_OK);
@@ -217,19 +221,23 @@ START_TEST(test_hoel_delete)
   ck_assert_int_eq(h_query_insert(conn, INSERT_DATA_2), H_OK);
   ck_assert_int_eq(h_query_select(conn, SELECT_DATA_1, &result), H_OK);
   ck_assert_int_eq(result.nb_rows, 1);
-  ck_assert_int_eq(result.nb_columns, 3);
+  ck_assert_int_eq(result.nb_columns, 4);
   ck_assert_int_eq(result.data[0][0].type, HOEL_COL_TYPE_INT);
   ck_assert_int_eq(((struct _h_type_int *)result.data[0][0].t_data)->value, 1);
-  ck_assert_int_eq(result.data[0][1].type, HOEL_COL_TYPE_TEXT);
-  ck_assert_str_eq(((struct _h_type_text *)result.data[0][1].t_data)->value, "value1");
+  ck_assert_int_eq(result.data[0][1].type, HOEL_COL_TYPE_DOUBLE);
+  ck_assert_double_eq(((struct _h_type_double *)result.data[0][1].t_data)->value, 4.2);
+  ck_assert_int_eq(result.data[0][2].type, HOEL_COL_TYPE_TEXT);
+  ck_assert_str_eq(((struct _h_type_text *)result.data[0][2].t_data)->value, "value1");
   ck_assert_int_eq(h_clean_result(&result), H_OK);
   ck_assert_int_eq(h_query_select(conn, SELECT_DATA_2, &result), H_OK);
   ck_assert_int_eq(result.nb_rows, 1);
-  ck_assert_int_eq(result.nb_columns, 3);
+  ck_assert_int_eq(result.nb_columns, 4);
   ck_assert_int_eq(result.data[0][0].type, HOEL_COL_TYPE_INT);
   ck_assert_int_eq(((struct _h_type_int *)result.data[0][0].t_data)->value, 2);
-  ck_assert_int_eq(result.data[0][1].type, HOEL_COL_TYPE_TEXT);
-  ck_assert_str_eq(((struct _h_type_text *)result.data[0][1].t_data)->value, "value2");
+  ck_assert_int_eq(result.data[0][1].type, HOEL_COL_TYPE_DOUBLE);
+  ck_assert_double_eq(((struct _h_type_double *)result.data[0][1].t_data)->value, 5.4);
+  ck_assert_int_eq(result.data[0][2].type, HOEL_COL_TYPE_TEXT);
+  ck_assert_str_eq(((struct _h_type_text *)result.data[0][2].t_data)->value, "value2");
   ck_assert_int_eq(h_clean_result(&result), H_OK);
   ck_assert_int_eq(h_query_delete(conn, DELETE_DATA_1), H_OK);
   ck_assert_int_eq(h_query_select(conn, SELECT_DATA_ALL, &result), H_OK);
@@ -248,17 +256,17 @@ START_TEST(test_hoel_json_insert)
 {
   struct _h_connection * conn;
   char * str_query = NULL;
-  json_t * j_query = json_pack("{sss{sisss{ss}}}",
+  json_t * j_query = json_pack("{sss{sisss{ss}sf}}",
                                "table",
                                "test_table",
                                "values",
-                                 "integer_col",
-                                 1,
-                                 "string_col",
-                                 "value1",
+                                 "integer_col", 1,
+                                 "string_col", "value1",
                                  "date_col",
                                    "raw",
-                                   "date('now')"), * j_result = NULL;
+                                   "date('now')",
+                                  "double_col", 4.2),
+          * j_result = NULL;
   conn = h_connect_sqlite(DEFAULT_BD_PATH);
   ck_assert_ptr_ne(conn, NULL);
   ck_assert_int_eq(h_insert(conn, j_query, &str_query), H_OK);
@@ -267,16 +275,16 @@ START_TEST(test_hoel_json_insert)
                       "table",
                       "test_table",
                       "where",
-                        "integer_col",
-                        1);
+                        "integer_col", 1);
   ck_assert_int_eq(h_select(conn, j_query, &j_result, NULL), H_OK);
   ck_assert_ptr_ne(j_result, NULL);
   ck_assert_int_eq(json_is_array(j_result), 1);
   ck_assert_int_eq(json_array_size(j_result), 1);
   ck_assert_str_eq(json_string_value(json_object_get(json_array_get(j_result, 0), "string_col")), "value1");
   ck_assert_int_eq(json_integer_value(json_object_get(json_array_get(j_result, 0), "integer_col")), 1);
+  ck_assert_double_eq(json_real_value(json_object_get(json_array_get(j_result, 0), "double_col")), 4.2);
   json_decref(j_result);
-  ck_assert_int_eq(o_strlen(str_query), o_strlen("INSERT INTO test_table (integer_col,string_col,date_col) VALUES (1,'value1',date('now'))"));
+  ck_assert_int_eq(o_strlen(str_query), o_strlen("INSERT INTO test_table (integer_col,string_col,date_col,double_col) VALUES (1,'value1',date('now'),4.200000)"));
   h_free(str_query);
   ck_assert_int_eq(h_delete(conn, j_query, NULL), H_OK);
   json_decref(j_query);
@@ -289,17 +297,16 @@ START_TEST(test_hoel_json_update)
 {
   struct _h_connection * conn;
   char * str_query = NULL;
-  json_t * j_query = json_pack("{sss{sisss{ss}}}",
+  json_t * j_query = json_pack("{sss{sisss{ss}sf}}",
                                "table",
                                "test_table",
                                "values",
-                                 "integer_col",
-                                 1,
-                                 "string_col",
-                                 "value1",
+                                 "integer_col", 1,
+                                 "string_col", "value1",
                                  "date_col",
                                    "raw",
-                                   "date('now')"),
+                                   "date('now')",
+                                  "double_col", 4.2),
          * j_result = NULL;
   conn = h_connect_sqlite(DEFAULT_BD_PATH);
   ck_assert_ptr_ne(conn, NULL);
@@ -309,8 +316,7 @@ START_TEST(test_hoel_json_update)
                       "table",
                       "test_table",
                       "where",
-                        "integer_col",
-                        1);
+                        "integer_col", 1);
   ck_assert_int_eq(h_select(conn, j_query, &j_result, NULL), H_OK);
   json_decref(j_query);
   ck_assert_ptr_ne(j_result, NULL);
@@ -318,31 +324,31 @@ START_TEST(test_hoel_json_update)
   ck_assert_int_eq(json_array_size(j_result), 1);
   ck_assert_str_eq(json_string_value(json_object_get(json_array_get(j_result, 0), "string_col")), "value1");
   ck_assert_int_eq(json_integer_value(json_object_get(json_array_get(j_result, 0), "integer_col")), 1);
+  ck_assert_double_eq(json_real_value(json_object_get(json_array_get(j_result, 0), "double_col")), 4.2);
   json_decref(j_result);
-  j_query = json_pack("{sss{ss}s{si}}",
+  j_query = json_pack("{sss{sssf}s{si}}",
                       "table",
                       "test_table",
                       "set",
-                        "string_col",
-                        "new value1",
+                        "string_col", "new value1",
+                        "double_col", 5.4,
                       "where",
-                        "integer_col",
-                        1);
+                        "integer_col", 1);
   ck_assert_int_eq(h_update(conn, j_query, &str_query), H_OK);
   json_decref(j_query);
-  ck_assert_int_eq(o_strlen(str_query), o_strlen("UPDATE test_table SET string_col='new value1' WHERE integer_col='1'"));
+  ck_assert_int_eq(o_strlen(str_query), o_strlen("UPDATE test_table SET string_col='new value1', double_col=5.400000 WHERE integer_col='1'"));
   h_free(str_query);
   j_query = json_pack("{sss{si}}",
                       "table",
                       "test_table",
                       "where",
-                        "integer_col",
-                        1);
+                        "integer_col", 1);
   ck_assert_int_eq(h_select(conn, j_query, &j_result, NULL), H_OK);
   ck_assert_ptr_ne(j_result, NULL);
   ck_assert_int_eq(json_array_size(j_result), 1);
   ck_assert_str_eq(json_string_value(json_object_get(json_array_get(j_result, 0), "string_col")), "new value1");
   ck_assert_int_eq(json_integer_value(json_object_get(json_array_get(j_result, 0), "integer_col")), 1);
+  ck_assert_double_eq(json_real_value(json_object_get(json_array_get(j_result, 0), "double_col")), 5.4);
   json_decref(j_result);
   ck_assert_int_eq(h_delete(conn, j_query, NULL), H_OK);
   json_decref(j_query);
@@ -355,33 +361,31 @@ START_TEST(test_hoel_json_delete)
 {
   struct _h_connection * conn;
   char * str_query = NULL;
-  json_t * j_query = json_pack("{sss{sisss{ss}}}",
+  json_t * j_query = json_pack("{sss{sisss{ss}sf}}",
                                "table",
                                "test_table",
                                "values",
-                                 "integer_col",
-                                 1,
-                                 "string_col",
-                                 "value1",
+                                 "integer_col", 1,
+                                 "string_col", "value1",
                                  "date_col",
                                    "raw",
-                                   "date('now')"),
+                                   "date('now')",
+                                 "double_col", 4.2),
          * j_result = NULL;
   conn = h_connect_sqlite(DEFAULT_BD_PATH);
   ck_assert_ptr_ne(conn, NULL);
   ck_assert_int_eq(h_insert(conn, j_query, NULL), H_OK);
   json_decref(j_query);
-  j_query = json_pack("{sss{sisss{ss}}}",
+  j_query = json_pack("{sss{sisss{ss}sf}}",
                       "table",
                       "test_table",
                       "values",
-                        "integer_col",
-                        2,
-                        "string_col",
-                        "value2",
+                        "integer_col", 2,
+                        "string_col", "value2",
                         "date_col",
                           "raw",
-                          "strftime('%s','2016-06-22 00:52:56')");
+                          "strftime('%s','2016-06-22 00:52:56')",
+                         "double_col", 5.4);
   ck_assert_int_eq(h_insert(conn, j_query, NULL), H_OK);
   json_decref(j_query);
   j_query = json_pack("{ss}",
@@ -397,8 +401,7 @@ START_TEST(test_hoel_json_delete)
                       "table",
                       "test_table",
                       "where",
-                        "integer_col",
-                        1);
+                        "integer_col", 1);
   ck_assert_int_eq(h_delete(conn, j_query, &str_query), H_OK);
   ck_assert_int_eq(o_strlen(str_query), o_strlen("DELETE FROM test_table WHERE integer_col='1'"));
   h_free(str_query);
@@ -412,6 +415,7 @@ START_TEST(test_hoel_json_delete)
   ck_assert_str_eq(json_string_value(json_object_get(json_array_get(j_result, 0), "string_col")), "value2");
   ck_assert_int_eq(json_integer_value(json_object_get(json_array_get(j_result, 0), "integer_col")), 2);
   ck_assert_int_eq(json_integer_value(json_object_get(json_array_get(j_result, 0), "date_col")), 1466556776);
+  ck_assert_double_eq(json_real_value(json_object_get(json_array_get(j_result, 0), "double_col")), 5.4);
   json_decref(j_result);
   j_query = json_pack("{ss}",
                       "table",
@@ -430,33 +434,31 @@ START_TEST(test_hoel_json_select)
 {
   struct _h_connection * conn;
   char * str_query = NULL;
-  json_t * j_query = json_pack("{sss{sisss{ss}}}",
+  json_t * j_query = json_pack("{sss{sisss{ss}sf}}",
                                "table",
                                "test_table",
                                "values",
-                                 "integer_col",
-                                 1,
-                                 "string_col",
-                                 "value1",
+                                 "integer_col", 1,
+                                 "string_col", "value1",
                                  "date_col",
                                    "raw",
-                                   "date('now')"),
+                                   "date('now')",
+                                 "double_col", 4.2),
          * j_result = NULL;
   conn = h_connect_sqlite(DEFAULT_BD_PATH);
   ck_assert_ptr_ne(conn, NULL);
   ck_assert_int_eq(h_insert(conn, j_query, NULL), H_OK);
   json_decref(j_query);
-  j_query = json_pack("{sss{sisss{ss}}}",
+  j_query = json_pack("{sss{sisss{ss}sf}}",
                       "table",
                       "test_table",
                       "values",
-                        "integer_col",
-                        2,
-                        "string_col",
-                        "value2",
+                        "integer_col", 2,
+                        "string_col", "value2",
                         "date_col",
                           "raw",
-                          "strftime('%s','2016-06-22 00:52:56')");
+                          "strftime('%s','2016-06-22 00:52:56')",
+                        "double_col", 5.4);
   ck_assert_int_eq(h_insert(conn, j_query, NULL), H_OK);
   json_decref(j_query);
   
@@ -468,6 +470,7 @@ START_TEST(test_hoel_json_select)
   ck_assert_int_eq(json_array_size(j_result), 2);
   ck_assert_str_eq(json_string_value(json_object_get(json_array_get(j_result, 0), "string_col")), "value1");
   ck_assert_int_eq(json_integer_value(json_object_get(json_array_get(j_result, 0), "integer_col")), 1);
+  ck_assert_double_eq(json_real_value(json_object_get(json_array_get(j_result, 0), "double_col")), 4.2);
   json_decref(j_query);
   json_decref(j_result);
   h_free(str_query);
@@ -476,13 +479,13 @@ START_TEST(test_hoel_json_select)
                       "table",
                       "test_table",
                       "where",
-                        "integer_col",
-                        1);
+                        "integer_col", 1);
   ck_assert_int_eq(h_select(conn, j_query, &j_result, &str_query), H_OK);
   ck_assert_int_eq(o_strlen(str_query), o_strlen("SELECT * FROM test_table WHERE integer_col='1'  "));
   ck_assert_int_eq(json_array_size(j_result), 1);
   ck_assert_str_eq(json_string_value(json_object_get(json_array_get(j_result, 0), "string_col")), "value1");
   ck_assert_int_eq(json_integer_value(json_object_get(json_array_get(j_result, 0), "integer_col")), 1);
+  ck_assert_double_eq(json_real_value(json_object_get(json_array_get(j_result, 0), "double_col")), 4.2);
   json_decref(j_query);
   json_decref(j_result);
   h_free(str_query);
@@ -491,13 +494,13 @@ START_TEST(test_hoel_json_select)
                       "table",
                       "test_table",
                       "where",
-                        "string_col",
-                        "value1");
+                        "string_col", "value1");
   ck_assert_int_eq(h_select(conn, j_query, &j_result, &str_query), H_OK);
   ck_assert_int_eq(o_strlen(str_query), o_strlen("SELECT * FROM test_table WHERE string_col='value1'  "));
   ck_assert_int_eq(json_array_size(j_result), 1);
   ck_assert_str_eq(json_string_value(json_object_get(json_array_get(j_result, 0), "string_col")), "value1");
   ck_assert_int_eq(json_integer_value(json_object_get(json_array_get(j_result, 0), "integer_col")), 1);
+  ck_assert_double_eq(json_real_value(json_object_get(json_array_get(j_result, 0), "double_col")), 4.2);
   json_decref(j_query);
   json_decref(j_result);
   h_free(str_query);
@@ -527,6 +530,7 @@ START_TEST(test_hoel_json_select)
   ck_assert_int_eq(json_array_size(j_result), 2);
   ck_assert_str_eq(json_string_value(json_object_get(json_array_get(j_result, 0), "string_col")), "value1");
   ck_assert_int_eq(json_integer_value(json_object_get(json_array_get(j_result, 0), "integer_col")), 1);
+  ck_assert_double_eq(json_real_value(json_object_get(json_array_get(j_result, 0), "double_col")), 4.2);
   json_decref(j_query);
   json_decref(j_result);
   h_free(str_query);
@@ -537,9 +541,9 @@ START_TEST(test_hoel_json_select)
                       "where",
                         "integer_col",
                           "operator",
-                          "raw",
-                          "value",
-                          ">6");
+                            "raw",
+                            "value",
+                            ">6");
   ck_assert_int_eq(h_select(conn, j_query, &j_result, &str_query), H_OK);
   ck_assert_int_eq(o_strlen(str_query), o_strlen("SELECT * FROM test_table WHERE integer_col >6  "));
   ck_assert_int_eq(json_array_size(j_result), 0);
@@ -564,6 +568,7 @@ START_TEST(test_hoel_json_select)
   ck_assert_int_eq(json_array_size(j_result), 2);
   ck_assert_str_eq(json_string_value(json_object_get(json_array_get(j_result, 0), "string_col")), "value1");
   ck_assert_int_eq(json_integer_value(json_object_get(json_array_get(j_result, 0), "integer_col")), 1);
+  ck_assert_double_eq(json_real_value(json_object_get(json_array_get(j_result, 0), "double_col")), 4.2);
   json_decref(j_query);
   json_decref(j_result);
   h_free(str_query);
